@@ -29,8 +29,8 @@ import numpy as np
 coeffs = {
     'metopb': {'ah': np.array([0.166, 0.183, 0.201]),
                'al': np.array([0.055, 0.061, 0.029]),
-               'bh': np.array([2.019, 1.476, 1.478]),
-               'bl': np.array([2.019, 1.476, 1.478]),
+               'bh': np.array([2.019, 1.476, 1.748]),
+               'bl': np.array([2.019, 1.476, 1.748]),
                'ch': np.array([-0.201, -0.137, -0.033]),
                'cl': np.array([-0.201, -0.137, -0.033]),
                'c_dark': np.array([39.70, 40.00, 40.30]),
@@ -405,6 +405,9 @@ coeffs = {
                'b1': np.array([1 - 0.0, 1 - 0.11187, 1 - 0.05991]),
                'b2': np.array([0.0, 0.00054668, 0.00024985])}}
 
+"""Source Patmos-X Coeffs: Version Tag 'y2017r1_sbaf'
+https://cimss.ssec.wisc.edu/patmosx/avhrr_cal.html"""
+
 
 class Calibrator(object):
 
@@ -439,12 +442,17 @@ def calibrate_solar(counts, chan, year, jday, spacecraft, corr=1):
     sth = (cal.ah[chan] * (100.0 + cal.bh[chan] * t
                            + cal.ch[chan] * t * t)) / 100.0
     if cal.c_s is not None:
-        return np.where(counts <= cal.c_s[chan],
+        refl = np.where(counts <= cal.c_s[chan],
                         (counts - cal.c_dark[chan]) * stl * corr,
                         ((cal.c_s[chan] - cal.c_dark[chan]) * stl
                          + (counts - cal.c_s[chan]) * sth) * corr)
     else:
-        return (counts - cal.c_dark[chan]) * stl * corr
+        refl = (counts - cal.c_dark[chan]) * stl * corr
+
+    # Mask negative reflectances
+    refl[refl < 0] = np.nan
+
+    return refl
 
 
 def calibrate_thermal(counts, prt, ict, space, line_numbers, channel, spacecraft):
@@ -555,4 +563,7 @@ def calibrate_thermal(counts, prt, ict, space, line_numbers, channel, spacecraft
     if chan == 0:
         bt = np.where((counts - new_space) >= 0, 0.0, bt)
 
+    # Mask values outside valid range
+    bt = np.where(np.logical_or(bt < 170.0, bt > 350.0), np.nan, bt)
+
     return bt
