@@ -23,8 +23,9 @@
 """The tests package.
 """
 
-from pygac.tests import test_calibrate_pod, test_slerp, test_calibrate_klm, \
-    test_pod, test_corrections, test_reader, test_io
+from pygac.tests import (test_calibrate_pod, test_slerp, test_calibrate_klm,
+                         test_pod, test_tsm, test_reader, test_io,
+                         test_angles, test_init)
 import unittest
 
 
@@ -33,7 +34,8 @@ def suite():
     """
     mysuite = unittest.TestSuite()
     tests = (test_slerp, test_calibrate_klm, test_calibrate_pod,
-             test_pod, test_corrections, test_reader, test_io)
+             test_pod, test_tsm, test_reader, test_io, test_angles,
+             test_init)
     for test in tests:
         mysuite.addTests(test.suite())
 
